@@ -3,9 +3,11 @@ package XML::XPathScript;
 use strict;
 use warnings;
 
-# $Revision: 124 $ - $Date: 2006-01-02 16:45:53 -0500 (Mon, 02 Jan 2006) $
 
-=pod "
+
+# $Revision: 164 $ - $Date: 2006-03-07 21:16:13 -0500 (Tue, 07 Mar 2006) $
+
+=pod 
 
 =head1 NAME
 
@@ -43,9 +45,6 @@ XML::XPathScript - a Perl framework for XML stylesheets
 
 =head1 DESCRIPTION
 
-This is the I<XML::XPathScript> stylesheet framework, part of the
-AxKit project at http://axkit.org/.
-
 XPathScript is a stylesheet language similar in many ways to XSLT (in
 concept, not in appearance), for transforming XML from one format to
 another (possibly HTML, but XPathScript also shines for non-XML-like
@@ -68,39 +67,15 @@ manage hundreds of different XML tags.
 
 =head1 STYLESHEET WRITER DOCUMENTATION
 
-=head2 Creating stylesheets
+If you are interested to write stylesheets, refers to the
+B<XML::XPathScript::Stylesheet> manpage. You might also want 
+to take a peek at the manpage of B<xpathscript>, a program 
+bundled with this module to perform XPathScript transformations
+via the command line. 
 
-See http://axkit.org/docs/xpathscript/guide.dkb for a head start.
-There you will learn how to markup the embedded dialect and fill in
-the template hash $t.
+=head1 STYLESHEET UTILITY METHODS 
 
-=head2 xpathscript Invocation
-
-This CPAN module is bundled with an "xpathscript" shell tool that
-is to be invoked like this:
-
-   xpathscript mydocument.xml mystylesheet.xps
-
-It will produce the resulting document on standard output. For more
-options, refer to xpathscript's man page.
-
-=head2 XPathScript methods available from within the stylesheet
-
-A number of callback functions are available from the stylesheet
-proper.  They apply against the current document and template hash,
-which are transparently passed back and forth as global variables (see
-L</Global variables>). They are defined in the
-L<XML::XPathScript::Processor> package, which is implicitly imported into
-all code written in the embedded stylesheet dialect.
-
-The following methods are also available to peek at the internal state
-of the XPathScript engine from within the stylesheet. Although C<<
-XML::XPathScript->current()->whatever() >> may be called from anywhere
-within the stylesheet (except a BEGIN or END block or similar), it is
-B<most unwise> to alter the state of the interpreter from within a
-C<testcode> block as the order of evaluation of the XML nodes is not
-specified. Better tweak the stylesheet globals (e.g. L</binmode>) once
-and for all at the beginning of the stylesheet.
+Those methods are meants to be used from within a stylesheet.
 
 =over
 
@@ -113,7 +88,7 @@ similar. B<Beware though> that using the return value for altering (as
 opposed to reading) stuff from anywhere except the stylesheet's top
 level is unwise.
 
-=cut "
+=cut
 
 sub current {
     unless (defined $XML::XPathScript::current) {
@@ -123,35 +98,41 @@ sub current {
     return $XML::XPathScript::current;
 }
 
-=item I<interpolating()>
+=item interpolation 
 
-=item I<interpolating($boolean)>
+        $XML::XPathScript::current->interpolation()
+        $XML::XPathScript::current->interpolation( $boolean )
 
 Gets (first call form) or sets (second form) the XPath interpolation
-boolean flag. If true, values set in C<< $template->{pre} >> and
-similar may contain expressions within braces, that will be
-interpreted as XPath expressions and substituted in place: for
-example, when interpolation is on, the following code
+boolean flag. If true, values set in C< pre > and C< post >
+may contain expressions within curly braces, that will be
+interpreted as XPath expressions and substituted in place.
 
-   $t->{'link'}{pre} = '<a href="{@url}">';
-   $t->{'link'}{post} = '</a>';
+For example, when interpolation is on, the following code
+
+    $template->set( link => { pre  => '<a href="{@url}">',
+                              post => '</a>'               } );
 
 is enough for rendering a C<< <link> >> element as an HTML hyperlink.
 The interpolation-less version is slightly more complex as it requires a
 C<testcode>:
 
-   $t->{'link'}{testcode} = sub {
+   sub link_testcode  {
       my ($currentnode, $t) = @_;
       my $url = findvalue('@url', $currentnode);
-      $t->{pre}="<a href='$url'>";
-      $t->{post}='</a>';
+      $t->set({ pre  => "<a href='$url'>",
+                post => "</a>"             });
 	  return DO_SELF_AND_KIDS();
    };
 
-Interpolation is on by default. A (now undocumented) global variable
-used to change the default to off, but don't do that.
+Interpolation is on by default. 
 
-=cut "
+=cut 
+
+sub interpolation {
+    my $self = shift;
+    $self->interpolating( @_ );
+}
 
 sub interpolating {
     my $self=shift;
@@ -163,6 +144,33 @@ sub interpolating {
 		    !$XML::XPathScript::DoNotInterpolate; # Obsolete, for compatibility:
 }
 
+=item interpolation_regex
+
+    $regex = $XML::XPathScript::curent->interpolation_regex()
+    $XML::XPathScript::curent->interpolation_regex( $regex )
+
+Gets or sets the regex to use for interpolation. The value to be 
+interpolated must be capture by $1. 
+
+By default, the interpolation regex is qr/{(.*?)}/.
+
+Example:
+
+    $XML::XPathScript::current->interpolation_regex( qr#\|(.*?)\|# );
+
+    $template->set( bird => { pre => '|@name| |@gender| |@type|' } );
+
+=cut
+
+sub interpolation_regex {
+    my $self = shift;
+
+    $self->{interpolation_regex} = shift if @_;
+
+    return $self->{interpolation_regex};
+}
+
+
 =pod "
 
 =item I<binmode()>
@@ -186,157 +194,6 @@ sub binmode {
 
 =back
 
-=head2 Stylesheet Guidelines
-
-Here are a few things to watch out for when coding stylesheets.
-
-=head3 The Unicode mess
-
-Unicode is a balucitherian character numbering standard, that strives
-to be a superset of all character sets currently in use by humans and
-computers. Going Unicode is therefore the way of the future, as it
-will guarantee compatibility of your applications with every character
-set on planet Earth: for this reason, all XML-compliant APIs
-(XML::XPathScript being no exception) should return Unicode strings in
-all their calls, regardless of the charset used to encode the XML
-document to begin with.
-
-The gotcha is, the brave Unicode world sells itself in much the same
-way as XML when it promises that you'll still be able to read your
-data back in 30 years: that will probably turn out to be true, but
-until then, you can't :-)
-
-Therefore, you as a stylesheet author will more likely than not need
-to do some wrestling with Unicode in Perl, XML::XPathScript or
-not. Here is a primer on how.
-
-=head4 Unicode, UTF-8 and Perl
-
-Unicode is B<not> a text file format: UTF-8 is. Perl, when doing
-Unicode, prefers to use UTF-8 internally.
-
-Unicode is a character numbering standard: that is, an abstract
-registry that associates unique integer numbers to a cast of thousands
-of characters. For example the "smiling face" is character number
-0x263a, and the thin space is 0x2009 (there is a URL to a Unicode
-character table in L</SEE ALSO>). Of course, this means that the
-8-bits- (or even, Heaven forbid, 7-bits-?)-per-character idea goes
-through the window this instant. Coding every character on 16 bits in
-memory is an option (called UTF-16), but not as simple an idea as it
-sounds: one would have to rewrite nearly every piece of C code for
-starters, and even then the Chinese aren't quite happy with "only"
-65536 character code points.
-
-Introducing UTF-8, which is a way of encoding Unicode character
-numbers (of any size) in an ASCII- and C-friendly way: all 127 ASCII
-characters (such as "A" or or "/" or ".", but I<not> the ISO-8859-1
-8-bit extensions) have the same encoding in both ASCII and UTF-8,
-including the null character (which is good for strcpy() and
-friends). Of course, this means that the other characters are rendered
-using I<several> bytes, for example "" is "é" in UTF-8. The result
-is therefore vaguely intelligible for a Western reader.
-
-=head4 Output to UTF-8 with XPathScript
-
-The programmer- and C-friendly characteristics of UTF-8 have made it
-the choice for dealing with Unicode in Perl. The interpreter maintains
-an "UTF8-tainted" bit on every string scalar it handles (much like
-what L<perlsec> does for untrusted data). Every function in
-XML::XPathScript returns a string with such bit set to true:
-therefore, producing UTF-8 output is straightforward and one does not
-have to take any special precautions in XPathScript.
-
-=head4 Output to a non-UTF-8 character set with XPathScript
-
-When L</binmode> is invoked from the stylesheet body, it signals that
-the stylesheet output should I<not> be UTF-8, but instead some
-user-chosen character encoding that XML::XPathScript cannot and will
-not know or care about. Calling C<<
-XML::XPathScript->current()->binmode() >> has the following
-consequences:
-
-=over 2
-
-=item *
-
-presence of this "UTF-8 taint" in the stylesheet output is now a fatal
-error. That is, whenever the result of a template evaluation is marked
-internally in Perl with the "this string is UTF-8" flag (as opposed to
-being treated by Perl as binary data without character meaning, see
-L</perlunicode>), L<XML::XPathScript::Processor/translate_node> will
-croak;
-
-=item *
-
-the stylesheet therefore needs to build an "unicode firewall". That
-is, C<testcode> blocks have to take input in UTF-8 (as per the XML
-standard, UTF-8 indeed is what will be returned by
-L<XML::XPathScript::Processor/findvalue> and such) and provide output in
-binary (in whatever character set is intended for the output), lest
-I<translate_node()> croaks as explained above. The L<Unicode::String>
-module comes in handy to the stylesheet writer to cast from UTF-8 to
-an 8-bit-per-character charset such as ISO 8859-1, while laundering
-Perl's internal UTF-8-string bit at the same time;
-
-=item *
-
-the appropriate voodoo is performed on the output filehandle(s) so
-that a spurious, final charset conversion will not happen at print()
-time under any locales, versions of Perl, or phases of moon.
-
-=back
-
-
-=head3 XPath scalar return values considered harmful
-
-XML::XPath calls such as I<findvalue()> return objects in an object
-class designed to map one of the types mandated by the XPath spec (see
-L<XML::XPath> for details). This is often not what a Perl programmer
-comes to expect (e.g. strings and numbers cannot be treated the
-same). There are some work-arounds built in XML::XPath, using operator
-overloading: when using those objects as strings (by concatenating
-them, using them in regular expressions etc.), they become strings,
-through a transparent call to one of their methods such as I<<
-->value() >>. However, we do not support this for a variety of reasons
-(from limitations in L</overload> to stylesheet compatibility between
-XML::XPath and XML::LibXML to Unicode considerations), and that is why
-our L</findvalue> and friends return a real Perl scalar, in violation
-of the XPath specification.
-
-On the other hand, L</findnodes> does return a list of objects in list
-context, and an I<XML::XPath::NodeSet> or I<XML::LibXML::NodeList>
-instance in scalar context, obeying the XPath specification in
-full. Therefore you most likely do not want to call I<findnodes()> in
-scalar context, ever: replace
-
-   my $attrnode = findnodes('@url',$xrefnode); # WRONG!
-
-with
-
-   my ($attrnode) = findnodes('@url',$xrefnode);
-
-
-=head3 Do not use DOM method calls, for they make stylesheets non-portable
-
-The I<findvalue()> such functions described in
-L<XML::XPathScript::Processor> are not the only way of extracting bits from
-the XML document. Objects passed as the first argument to the I<<
-->{testcode} >> templates and returned by I<findnodes()> in array
-context are of one of the I<XML::XPath::Node::*> classes, and they
-feature some data extraction methods by themselves, conforming to the
-DOM specification.
-
-However, the names of those methods are not standardized even among
-DOM parsers (the accessor to the C<childNodes> property, for example,
-is named C<childNodes()> in I<XML::LibXML> and C<getChildNodes()> in
-I<XML::XPath>!). In order to write a stylesheet that is portable
-between L<XML::libXML> and L<XML::XPath> used as back-ends to
-L<XML::XPathScript>, one should refrain from doing that. The exact
-same data is available through appropriate XPath formulae, albeit more
-slowly, and there are also type-checking accessors such as
-C<is_element_node()> in L<XML::XPathScript::Processor>.
-
-
 =head1 TECHNICAL DOCUMENTATION
 
 The rest of this POD documentation is B<not> useful to programmers who
@@ -368,77 +225,20 @@ directed by the stylesheet.
 
 =back
 
-When run, the stylesheet is expected to fill in the I<template hash>
-$t, which is a lexically-scoped variable made available to it at
+When run, the stylesheet is expected to fill in the I<template object>
+$template, which is a lexically-scoped variable made available to it at
 preprocess time.
 
-=head2 Dependencies
-
-Although XPathScript is a core component of AxKit, which depends on
-this module to be able to process XPathScript stylesheets, there is
-plenty of motivation for doing stylesheets outside of a WWW
-application server and so I<XML::XPathScript> is also distributed as a
-standalone CPAN module.  The AxKit XPathScript component inherits from
-this class and provides the coupling with the application framework by
-overloading and adding some methods.
-
-I<XML::XPathScript> requires the following Perl packages:
-
-=over
-
-=item I<Symbol>
-
-For loading files from anonymous filehandles. I<Symbol> is bundled
-with Perl.
-
-=item I<File::Basename>
-
-For fetching stylesheets from system files. One may provide other
-means of fetching stylesheets through object inheritance (this is what
-AxKit does). I<File::Basename> is bundled with Perl.
-
-=item I<XML::Parser>
-
-=item I<XML::XPath>
-
-For the XML parser and XPath interpreter, obviously needed. Plans are
-to support the I<XML::libXML> package as an alternative, which does
-the same as the above in C (and hence an order of magnitude faster).
-
-=back
-
-=head2 Global variables
-
-Due to the peculiar syntax allowed in the embedded dialect for
-accessing the template hash, the stylesheet is not reentrant and
-cannot (yet) transform several documents at once. However, one should
-not rely on those variables existing forever.
-
-=over
-
-=item I<$XML::XPathScript::xp>
-
-The XML::XPath object that holds the whole document (created by
-L<XML::XPath/new>)
-
-=item I<$XML::XPathScript::trans>
-
-The template hash currently in use (known as $t in the AxKit
-documentation). Its keys are element names, and its values are
-the matching templates (as hash references).
-
-=back
-
 =cut "
 
-
 use vars qw( $VERSION $XML_parser $DoNotInterpolate $debug_level );
 
 use Symbol;
 use File::Basename;
 use XML::XPathScript::Processor;
+use XML::XPathScript::Template;
 
-$VERSION = '0.16';
+$VERSION = '1.00';
 
 $XML_parser = 'XML::LibXML';
 
@@ -451,16 +251,14 @@ $debug_level = 0;
 
 sub import
 {
-	my $self = shift @_;
+    my $self = shift @_;
 
-	if( grep $_ eq 'XML::XPath', @_ )
-	{
-		$XML::XPathScript::XML_parser = 'XML::XPath';
-	}
-	elsif( grep $_ eq 'XML::LibXML', @_ )
-	{
-		$XML::XPathScript::XML_parser = 'XML::LibXML';
-	}
+    if ( grep $_ eq 'XML::XPath', @_ ) {
+        $XML::XPathScript::XML_parser = 'XML::XPath';
+    }
+    elsif ( grep $_ eq 'XML::LibXML', @_ ) {
+        $XML::XPathScript::XML_parser = 'XML::LibXML';
+    }
 }
 
 INIT
@@ -486,7 +284,7 @@ EOT
 
 =pod "
 
-=head2 Methods and class methods
+=head1 METHODS
 
 =over
 
@@ -500,9 +298,7 @@ Creates a new XPathScript translator. The recognized named arguments are
 
 $xml is a scalar containing XML text, or a reference to a filehandle
 from which XML input is available, or an I<XML::XPath> or
-I<XML::libXML> object (support for the latter object class is very
-poor for now, as it involves unparsing and parsing back into
-I<XML::XPath>).
+I<XML::libXML> object.
 
 An XML::XPathscript object without an I<xml> argument
 to the constructor is only able to compile stylesheets (see
@@ -570,8 +366,8 @@ or
 
    $xps->process(sub {print ANOTHERFD (shift);});
 
-(not that the latter would be any good, since C<<
-$xps->process(\*ANOTHERFD) >> would do exactly the same, only faster)
+(not that the latter would be any good, since 
+C< $xps->process(\*ANOTHERFD) > would do exactly the same, only faster)
 
 If the stylesheet was I<compile()>d with extra I<varname>s, then the
 calling code should call I<process()> with a corresponding number of
@@ -734,10 +530,17 @@ sub extract {
 
     my $contents = $self->read_stylesheet( $stylesheet );
 
+    # <%- -%> magic
+    $contents =~ s#(\s+)<%-([=~]?)#<%$2$1#gs;
+    $contents =~ s#-%>(\s+)#$1%>#gs;
+
+    # <%~ %> magic
+    $contents =~ s#<%~\s+(\S+)\s+%>#<%= apply_templates( qq<$1> ) %>#gs;
+
     my $script="#line 1 $filename\n",
     my $line = 1;
 
-    while ($contents =~ /\G(.*?)(<!--#include|<%[#=]?)/gcs) {
+    while ($contents =~ /\G(.*?)(<!--#include|<%[=#]?)/gcs) {
         my ($text, $type) = ($1, $2);
         $line += $text =~ tr/\n//; # count \n's in text
         $text =~ s/\|/\\\|/g;
@@ -874,9 +677,7 @@ sub include_file {
 =item I<compile(varname1, varname2,...)>
 
 Compiles the stylesheet set at I<new()> time and returns an anonymous
-CODE reference. $stylesheet shall be written in the unparsed embedded
-dialect (in other words C<< ->extract($stylesheet) >> will be called
-first inside I<compile()>).
+CODE reference. 
 
 I<varname1>, I<varname2>, etc. are extraneous arguments that will be
 made available to the stylesheet dialect as lexically scoped
@@ -939,7 +740,9 @@ sub compile {
 		    sub {
 		    	my (\$self, $extravars ) = \@_;
 				local \$XML::XPathScript::current=\$self;
-		    	my \$t = \$XML::XPathScript::current->{t} = {};
+		    	my \$t = \$XML::XPathScript::current->{t} 
+                            = XML::XPathScript::Template->new();
+                my \$template = \$t;
 				local \$XML::XPathScript::trans = \$t; # Yes,
 				# this does the sharing! Perl is a bizarre and
 				# wonderful language.
@@ -963,8 +766,8 @@ EOT
 =item I<print($text)>
 
 Outputs a chunk of text on behalf of the stylesheet. The default
-implementation is to use the second argument to L</process>, which was
-stashed in C<< $self->{printer} >> by said function. Overloading this
+implementation is to use the second argument to L</process>. 
+Overloading this
 method in a subclass provides yet another method to redirect output.
 
 =cut "
@@ -996,9 +799,7 @@ sub debug {
 
 =back
 
-=head2 Utility functions
-
-The functions below are not methods.
+=head1 FUNCTIONS
 
 =over
 
@@ -1019,7 +820,7 @@ sub gen_package_name {
 
 =item  $nodeset = $xps->document( $uri )
 
-	Reads XML given in $uri, parses it and returns it in a nodeset.
+Reads XML given in $uri, parses it and returns it in a nodeset.
 
 =cut
 
@@ -1108,11 +909,11 @@ __END__
 
 =head1 AUTHORS
 
-Created by Matt Sergeant <matt@sergeant.org>
+Current maintainers: 
+Yanick Champoux <yanick@cpan.org> 
+and Dominique Quatravaux <dom@idealx.com>
 
-Improvements and feature merge with
-Apache::AxKit::Language::XPathScript by Yanick Champoux
-<yanick@babyl.dyndns.org> and Dominique Quatravaux <dom@idealx.com>
+Created by Matt Sergeant <matt@sergeant.org>
 
 =head1 LICENSE
 
@@ -1121,17 +922,17 @@ Perl itself.
 
 =head1 SEE ALSO
 
-The XPathScript Guide at
+L<XML::XPathScript::Stylesheet>, L<XML::XPathScript::Processor>, 
+L<XML::XPathScript::Template>, L<XML::XPathScript::Template::Tag>
 
-  http://axkit.org/wiki/view/AxKit/XPathScriptGuide
+Guide of the original Axkit XPathScript: 
+http://axkit.org/wiki/view/AxKit/XPathScriptGuide
 
 XPath documentation from W3C:
-
-  http://www.w3.org/TR/xpath
+http://www.w3.org/TR/xpath
 
 Unicode character table:
-
-  http://www.unicode.org/charts/charindex.html
+http://www.unicode.org/charts/charindex.html
 
 =cut
 
